(function(w, $) {
	var audioPlayer = function(audioId, options, callback) {
			var self = this,
				cssSelector = {
					play: ".ap-btn-play",
					pause: ".ap-btn-pause",
					prev: ".ap-btn-prev",
					next: ".ap-btn-next",
					infoTitle: ".ap-info-title",
					currentTime: ".ap-current-time",
					duration: ".ap-duration-time",
					progress: ".ap-progress",
					progressBar: ".ap-progress-bar",
					bufferBar: ".ap-progress-buffer",
					playBar: ".ap-progress-play",
					playBarBtn: ".ap-progress-handle"
				},
				defaults = {
					autoPlay: false,
					playMode: 0,
					currentIndex: 0,
					liveState: 0,
					selectorMode: 0
				},
				audio = $(audioId);
			this.ua = window.navigator.userAgent;
			this.options = $.extend(true, {}, defaults, cssSelector, options);
			this.playList = this.options.playList;
			this.currentPlay = this.options.currentPlay;
			this.callback = callback;
			this.jq = {
				audio: $(audioId)
			};
			!self.options.selectorMode && self._initBody();
			$.each(cssSelector, function(entity, cssSel) {
				self.jq[entity] = $(options.cssSelectorAncestor + " " + cssSel)
			});
			this.dom = {
				audio: $(audioId)[0]
			};
			$.each(this.jq, function(entity, cssSel) {
				self.dom[entity] = cssSel[0]
			});
			setTimeout(function() {
				self._initPlayer();
				self._sliderBar()
			}, 100)
		},
		_getTime = function(milliseconds) {
			if (milliseconds == "Infinity") {
				return "00:00"
			}
			var seconds = Math.floor(milliseconds),
				minutes = Math.floor(seconds / 60);
			seconds = seconds - (minutes * 60);
			return (minutes < 10 ? "0" + minutes : minutes) + ":" + (seconds < 10 ? "0" + seconds : seconds)
		},
		getAbsPoint = function(e) {
			var obj = {
				x: e.offsetLeft,
				y: e.offsetTop
			};
			while (e = e.offsetParent) {
				obj.x += e.offsetLeft;
				obj.y += e.offsetTop
			}
			return obj
		};
	audioPlayer.prototype = {
		progressState: true,
		_initPlayer: function() {
			var self = this;
			this.progressWidth = this.jq.progress.width();
			this._setInfo();
			this._setSrc();
			this.jq.play.on("click", function(event) {
				var _this = this;
				self.play();
				$(this).addClass("active");
				setTimeout(function() {
					$(_this).css({
						display: "none"
					}).removeClass("active");
					self.jq.pause.css({
						display: "block"
					})
				}, 100);
				self.callback && self.callback(self)
			});
			this.jq.pause.on("click", function(event) {
				var _this = this;
				self.pause();
				$(this).addClass("active");
				setTimeout(function() {
					$(_this).css({
						display: "none"
					}).removeClass("active");
					self.jq.play.css({
						display: "block"
					})
				}, 100);
				self.callback && self.callback(self)
			});
			this.jq.next.on("click", function(event) {
				var _this = this;
				self.currentIndex++;
				if (self.currentIndex >= self.playList.length - 1) {
					self.currentIndex = self.playList.length - 1
				}
				var obj = self.playList[self.currentIndex];
				self.setFile(obj, true);
				$(this).addClass("active");
				setTimeout(function() {
					$(_this).removeClass("active")
				}, 100)
			});
			this.jq.prev.on("click", function(event) {
				var _this = this;
				self.currentIndex--;
				if (self.currentIndex <= 0) {
					self.currentIndex = 0
				}
				var obj = self.playList[self.currentIndex];
				self.setFile(obj, true);
				$(this).addClass("active");
				setTimeout(function() {
					$(_this).removeClass("active")
				}, 100)
			});
			if (this.dom.audio != undefined) {
				this.dom.audio.addEventListener("loadstart", function() {}, false);
				this.dom.audio.addEventListener("canplay", function(event) {
					var duration = _getTime(this.duration);
					self.jq.duration.html(duration);
					if (!this.paused) {
						return false
					}
					self.jq.currentTime.html("00:00");
					self.jq.playBarBtn.css({
						left: 0
					});
					self.jq.playBar.css({
						width: 0
					})
				}, false);
				this.dom.audio.addEventListener("timeupdate", function(event) {
					var currentTime = _getTime(this.currentTime),
						duration = _getTime(this.duration);
					if (self.progressState) {
						self.jq.currentTime.text(currentTime)
					}
					if (!this.paused && this.currentTime > 0 && self.progressState) {
						var percent = (this.currentTime / this.duration) * 100;
						self.jq.playBar.css({
							width: percent + "%"
						});
						self.jq.playBarBtn.css({
							left: percent + "%"
						})
					}
					if (!this.paused && this.currentTime > 0 && this.readyState == 4) {
						self.jq.play.css({
							display: "none"
						}).removeClass("active");
						self.jq.pause.css({
							display: "block"
						})
					}
				}, false);
				this.dom.audio.addEventListener("progress", function(event) {
					var percent = 0;
					if ((typeof this.buffered === "object") && (this.buffered.length > 0)) {
						if (this.duration > 0) {
							var bufferTime = 0;
							for (var i = 0; i < this.buffered.length; i++) {
								bufferTime += this.buffered.end(i) - this.buffered.start(i)
							}
							percent = 100 * bufferTime / this.duration
						}
					} else {
						percent = 0
					}
					self.jq.bufferBar.css({
						width: percent + "%"
					})
				}, false);
				this.dom.audio.addEventListener("ended", function(event) {
					self.jq.currentTime.text("00:00");
					self.jq.playBarBtn.css({
						left: 0
					});
					self.jq.playBar.css({
						width: 0
					});
					if (self.options.playMode == 1) {
						self.jq.next.trigger("click");
						self.play()
					} else {
						setTimeout(function() {
							self.jq.pause.css({
								display: "none"
							});
							self.jq.play.css({
								display: "block"
							})
						}, 100)
					}
				}, false)
			}
			this._autoPlay()
		},
		_initBody: function() {
			var self = this,
				c = $(self.options.cssSelectorAncestor),
				b = $('<div class="ap-btn"><a href="javascript:;" class="ap-btn-play"></a><a href="javascript:;" class="ap-btn-pause"></a></div>'),
				d = $('<div class="ap-detail" />'),
				i = $('<div class="ap-info"><div class="ap-info-title"></div></div>'),
				s = $('<div class="ap-progress"><div class="ap-progress-bar"><div class="ap-progress-play"></div><div class="ap-progress-buffer"></div></div><div class="ap-progress-handle"></div></div>'),
				m = $('<div class="ap-timer"><span class="ap-current-time">00:00</span><span class="ap-duration-time">00:00</span></div>');
			d.append(i), d.append(s), d.append(m), b.appendTo(c), d.appendTo(c)
		},
		setFile: function(obj, autoplay) {
			this.currentPlay = obj;
			if (this.currentPlay != null && this.currentPlay.src) {
				this.dom.audio.src = this.currentPlay.src
			}
			autoplay && this.jq.play.trigger("click");
			this.callback && this.callback(this);
			this.currentIndex = this.currentPlay.index;
			this._setInfo()
		},
		play: function() {
			this.dom.audio.play()
		},
		pause: function() {
			this.dom.audio.pause()
		},
		_cssName: function(n) {},
		_setInfo: function() {
			this.jq.infoTitle.text(this.currentPlay.title)
		},
		_setSrc: function() {
			if (this.currentPlay.src) {
				this.dom.audio.src = this.currentPlay.src
			} else if (this.playList.length) {
				this.dom.audio.src = this.playList[0].src
			}
			this.currentIndex = this.currentPlay.index
		},
		_autoPlay: function() {
			if(this.options.autoPlay) {
				this.dom.audio.autoplay = true;
				this.jq.play.trigger("click")
			}else{
				this.dom.audio.autoplay = false;
				this.jq.pause.trigger("click")
			}
		},
		_sliderBar: function() {
			var self = this,
				dx = 0,
				cx = 0,
				curFlag = false,
				currentTime = 0,
				percent = 0;
			var btnStart = function(event) {
					var _this = event.data.context,
						type = event.data.type,
						ev = event;
					if (type == "touch") {
						ev = event.targetTouches[0]
					}
					curFlag = true;
					dx = ev.clientX - _this.offsetLeft;
					event.preventDefault()
				};
			var btnMove = function(event) {
					var _this = event.data.context,
						type = event.data.type,
						ev = event;
					if (type == "touch") {
						ev = event.targetTouches[0]
					}
					self.progressState = false;
					if (curFlag) {
						cx = ev.clientX - dx;
						if (cx <= 0) {
							cx = 0
						} else if (cx >= self.progressWidth) {
							cx = self.progressWidth
						}
						currentTime = ((self.dom.audio.duration / self.progressWidth) * cx);
						percent = (cx / self.progressWidth) * 100;
						self.jq.currentTime.text(_getTime(currentTime));
						self.jq.playBar.css({
							width: percent + "%"
						});
						$(_this).css({
							left: percent + "%"
						})
					};
					event.preventDefault()
				};
			var btnEnd = function(event) {
					var _this = event.data.context,
						type = event.data.type,
						ev = event;
					self.dom.audio.currentTime = currentTime;
					self.dom.audio.paused && self.play();
					if (type == "touch") {
						ev = event.targetTouches[0];
						_this.ontouchmove = null;
						_this.ontouchend = null
					} else {
						_this.onmousemove = null;
						_this.onmouseend = null
					}
					curFlag = false;
					self.progressState = true;
					event.preventDefault()
				};
			var data = {
				context: this.dom.playBarBtn,
				type: "touch"
			};
			this.jq.playBarBtn.on("touchstart", data, btnStart);
			this.jq.playBarBtn.on("touchmove", data, btnMove);
			this.jq.playBarBtn.on("touchend", data, btnEnd);
			var _dx = 0,
				_currentTime = 0,
				_percent = 0;
			var progressHandle = function(event) {
					var _this = event.data.context,
						type = event.data.type,
						ev = event;
					if (ev.target.className == "ap-progress-handle") {
						return
					}
					if (type == "touch") {
						ev = event.changedTouches[0]
					}
					var selfPonit = getAbsPoint(_this);
					_dx = ev.clientX - selfPonit.x;
					_percent = ((ev.clientX - selfPonit.x) / self.progressWidth) * 100;
					if (_dx <= 0 || _percent <= 0) {
						_dx = _percent = 0
					} else if (_dx >= self.progressWidth) {
						_dx = self.progressWidth;
						_percent = 100
					}
					_currentTime = ((self.dom.audio.duration / self.progressWidth) * _dx);
					self.dom.audio.currentTime = _currentTime;
					self.jq.playBarBtn.css({
						left: _percent + "%"
					});
					self.jq.playBar.css({
						width: _percent + "%"
					});
					_this.ontouchend = null;
					event.preventDefault()
				};
			this.jq.progressBar.on("touchend", {
				context: this.dom.progressBar,
				type: "touch"
			}, progressHandle)
		}
	};
	w.audioPlayer = audioPlayer
})(window, $);